use hyper::server::conn::http1;
use hyper_util::rt::TokioIo;
use tokio::net::TcpListener;

mod sni;
mod env;

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error + Send + Sync>> {
    let env = env::Env::default();

    println!("Running on {}", env.listen);

    let listener = TcpListener::bind(env.listen).await?;

    loop {
        let (stream, _) = listener.accept().await?;

        let io = TokioIo::new(stream);
        let svc = sni::SniWebsitesService {};

        tokio::task::spawn(async move {
            if let Err(e) = http1::Builder::new()
                .serve_connection(io, svc)
                .await {
                    eprintln!("Error while serving connection:");
                    eprintln!("{e:#?}");
            }
        });
    }
}
